@@ -46,7 +46,7 @@
 
 #include <telepathy-logger/log-manager.h>
 
-#include <libempathy/empathy-idle.h>
+#include <libempathy/empathy-presence-manager.h>
 #include <libempathy/empathy-utils.h>
 #include <libempathy/empathy-chatroom-manager.h>
 #include <libempathy/empathy-account-settings.h>
@@ -63,9 +63,9 @@
 #include "empathy-main-window.h"
 #include "empathy-accounts-common.h"
 #include "empathy-accounts-dialog.h"
-#include "empathy-chat-manager.h"
 #include "empathy-status-icon.h"
 #include "empathy-ft-manager.h"
+#include "empathy-notifications-approver.h"
 
 #include "extensions/extensions.h"
 
@@ -105,6 +105,8 @@ struct _EmpathyApp
   gboolean no_connect;
   gboolean start_hidden;
 
+  gboolean activated;
+
   GtkWidget *window;
   EmpathyStatusIcon *icon;
   EmpathyDispatcher *dispatcher;
@@ -112,10 +114,10 @@ struct _EmpathyApp
   TplLogManager *log_manager;
   EmpathyChatroomManager *chatroom_manager;
   EmpathyFTFactory  *ft_factory;
-  EmpathyIdle *idle;
+  EmpathyPresenceManager *presence_mgr;
   EmpathyConnectivity *connectivity;
-  EmpathyChatManager *chat_manager;
   GSettings *gsettings;
+  EmpathyNotificationsApprover *notifications_approver;
 #ifdef HAVE_GEOCLUE
   EmpathyLocationManager *location_manager;
 #endif
@@ -134,17 +136,17 @@ empathy_app_dispose (GObject *object)
   void (*dispose) (GObject *) =
     G_OBJECT_CLASS (empathy_app_parent_class)->dispose;
 
-  if (self->idle != NULL)
+  if (self->presence_mgr != NULL)
     {
-      empathy_idle_set_state (self->idle, TP_CONNECTION_PRESENCE_TYPE_OFFLINE);
+      empathy_presence_manager_set_state (self->presence_mgr,
+          TP_CONNECTION_PRESENCE_TYPE_OFFLINE);
     }
 
 #ifdef ENABLE_DEBUG
   tp_clear_object (&self->debug_sender);
 #endif
 
-  tp_clear_object (&self->chat_manager);
-  tp_clear_object (&self->idle);
+  tp_clear_object (&self->presence_mgr);
   tp_clear_object (&self->connectivity);
   tp_clear_object (&self->icon);
   tp_clear_object (&self->account_manager);
@@ -156,6 +158,7 @@ empathy_app_dispose (GObject *object)
 #endif
   tp_clear_object (&self->ft_factory);
   tp_clear_object (&self->gsettings);
+  tp_clear_object (&self->notifications_approver);
 
   if (dispose != NULL)
     dispose (object);
@@ -168,7 +171,8 @@ empathy_app_finalize (GObject *object)
   void (*finalize) (GObject *) =
     G_OBJECT_CLASS (empathy_app_parent_class)->finalize;
 
-  gtk_widget_destroy (self->window);
+  if (self->window != NULL)
+    gtk_widget_destroy (self->window);
 
   if (finalize != NULL)
     finalize (object);
@@ -178,36 +182,6 @@ static void account_manager_ready_cb (GObject *source_object,
     GAsyncResult *result,
     gpointer user_data);
 
-static EmpathyApp *
-empathy_app_new (guint argc,
-    const gchar * const * argv,
-    gboolean no_connect,
-    gboolean start_hidden)
-{
-  EmpathyApp *self;
-  GError *error = NULL;
-  GVariant *argv_variant;
-
-  argv_variant = g_variant_new_bytestring_array (argv, argc);
-
-  self = g_initable_new (EMPATHY_TYPE_APP,
-      NULL, &error,
-      "application-id", EMPATHY_DBUS_NAME,
-      "argv", argv_variant,
-      "register", TRUE,
-      "no-connect", no_connect,
-      "start-hidden", start_hidden,
-      NULL);
-
-  if (self == NULL)
-    {
-      g_critical ("Failed to initiate EmpathyApp: %s", error->message);
-      g_error_free (error);
-    }
-
-  return self;
-}
-
 static void
 empathy_app_set_property (GObject *object,
     guint prop_id,
@@ -231,21 +205,145 @@ empathy_app_set_property (GObject *object,
 }
 
 static void
-empathy_app_activated (GtkApplication *app,
-    GVariant *args)
+new_incoming_transfer_cb (EmpathyFTFactory *factory,
+    EmpathyFTHandler *handler,
+    GError *error,
+    gpointer user_data)
+{
+  if (error)
+    empathy_ft_manager_display_error (handler, error);
+  else
+    empathy_receive_file_with_file_chooser (handler);
+}
+
+static void
+new_ft_handler_cb (EmpathyFTFactory *factory,
+    EmpathyFTHandler *handler,
+    GError *error,
+    gpointer user_data)
+{
+  if (error)
+    empathy_ft_manager_display_error (handler, error);
+  else
+    empathy_ft_manager_add_handler (handler);
+
+  g_object_unref (handler);
+}
+
+static int
+empathy_app_command_line (GApplication *app,
+    GApplicationCommandLine *cmdline)
 {
   EmpathyApp *self = (EmpathyApp *) app;
 
-  /* We're requested to show stuff again, disable the start hidden global
-   * in case the accounts wizard wants to pop up.
-   */
-  self->start_hidden = FALSE;
+  if (!self->activated)
+    {
+      GError *error = NULL;
 
-  empathy_window_present (GTK_WINDOW (self->window));
+      /* Create the FT factory */
+      self->ft_factory = empathy_ft_factory_dup_singleton ();
+      g_signal_connect (self->ft_factory, "new-ft-handler",
+          G_CALLBACK (new_ft_handler_cb), NULL);
+      g_signal_connect (self->ft_factory, "new-incoming-transfer",
+          G_CALLBACK (new_incoming_transfer_cb), NULL);
+
+      if (!empathy_ft_factory_register (self->ft_factory, &error))
+        {
+          g_warning ("Failed to register FileTransfer handler: %s",
+              error->message);
+          g_error_free (error);
+        }
+
+      g_application_hold (G_APPLICATION (app));
+      self->activated = TRUE;
+
+      /* Setting up UI */
+      self->window = empathy_main_window_dup ();
+      gtk_widget_show (self->window);
+      self->icon = empathy_status_icon_new (GTK_WINDOW (self->window),
+          self->start_hidden);
+
+      self->notifications_approver =
+        empathy_notifications_approver_dup_singleton ();
+    }
+  else
+    {
+      /* We're requested to show stuff again, disable the start hidden global in
+       * case the accounts wizard wants to pop up.
+       */
+      self->start_hidden = FALSE;
+    }
+
+  if (!self->start_hidden)
+    empathy_window_present (GTK_WINDOW (self->window));
 
   /* Display the accounts dialog if needed */
   tp_account_manager_prepare_async (self->account_manager, NULL,
       account_manager_ready_cb, self);
+
+  return 0;
+}
+
+static gboolean
+show_version_cb (const char *option_name,
+    const char *value,
+    gpointer data,
+    GError **error);
+
+static gboolean
+empathy_app_local_command_line (GApplication *app,
+    gchar ***arguments,
+    gint *exit_status)
+{
+  EmpathyApp *self = (EmpathyApp *) app;
+  gint i;
+  gchar **argv;
+  gint argc = 0;
+  gboolean retval = FALSE;
+  GError *error = NULL;
+  gboolean no_connect = FALSE, start_hidden = FALSE;
+
+  GOptionContext *optcontext;
+  GOptionEntry options[] = {
+      { "no-connect", 'n',
+        0, G_OPTION_ARG_NONE, &no_connect,
+        N_("Don't connect on startup"),
+        NULL },
+      { "start-hidden", 'h',
+        0, G_OPTION_ARG_NONE, &start_hidden,
+        N_("Don't display the contact list or any other dialogs on startup"),
+        NULL },
+      { "version", 'v',
+        G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, show_version_cb,
+        NULL, NULL },
+      { NULL }
+  };
+
+  optcontext = g_option_context_new (N_("- Empathy IM Client"));
+  g_option_context_add_group (optcontext, gtk_get_option_group (TRUE));
+  g_option_context_add_main_entries (optcontext, options, GETTEXT_PACKAGE);
+
+  argv = *arguments;
+  for (i = 0; argv[i] != NULL; i++)
+    argc++;
+
+  if (!g_option_context_parse (optcontext, &argc, &argv, &error))
+    {
+      g_print ("%s\nRun '%s --help' to see a full list of available command "
+          "line options.\n",
+          error->message, argv[0]);
+      g_warning ("Error in empathy init: %s", error->message);
+
+      *exit_status = EXIT_FAILURE;
+      retval = TRUE;
+    }
+
+  g_option_context_free (optcontext);
+
+  self->no_connect = no_connect;
+  self->start_hidden = start_hidden;
+
+  return retval;
 }
 
 static void empathy_app_constructed (GObject *object);
@@ -254,7 +352,7 @@ static void
 empathy_app_class_init (EmpathyAppClass *klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
-  GtkApplicationClass *gtk_app_class = GTK_APPLICATION_CLASS (klass);
+  GApplicationClass *g_app_class = G_APPLICATION_CLASS (klass);
   GParamSpec *spec;
 
   gobject_class->set_property = empathy_app_set_property;
@@ -262,7 +360,8 @@ empathy_app_class_init (EmpathyAppClass *klass)
   gobject_class->dispose = empathy_app_dispose;
   gobject_class->finalize = empathy_app_finalize;
 
-  gtk_app_class->activated = empathy_app_activated;
+  g_app_class->command_line = empathy_app_command_line;
+  g_app_class->local_command_line = empathy_app_local_command_line;
 
   spec = g_param_spec_boolean ("no-connect", "no connect",
       "Don't connect on startup",
@@ -381,32 +480,6 @@ show_version_cb (const char *option_name,
   exit (EXIT_SUCCESS);
 }
 
-static void
-new_incoming_transfer_cb (EmpathyFTFactory *factory,
-    EmpathyFTHandler *handler,
-    GError *error,
-    gpointer user_data)
-{
-  if (error)
-    empathy_ft_manager_display_error (handler, error);
-  else
-    empathy_receive_file_with_file_chooser (handler);
-}
-
-static void
-new_ft_handler_cb (EmpathyFTFactory *factory,
-    EmpathyFTHandler *handler,
-    GError *error,
-    gpointer user_data)
-{
-  if (error)
-    empathy_ft_manager_display_error (handler, error);
-  else
-    empathy_ft_manager_add_handler (handler);
-
-  g_object_unref (handler);
-}
-
 static void
 account_manager_ready_cb (GObject *source_object,
     GAsyncResult *result,
@@ -448,7 +521,7 @@ account_manager_ready_cb (GObject *source_object,
           (presence, TP_CONNECTION_PRESENCE_TYPE_OFFLINE)
             <= 0)
       /* if current state is Offline, then put it online */
-      empathy_idle_set_state (self->idle,
+      empathy_presence_manager_set_state (self->presence_mgr,
           TP_CONNECTION_PRESENCE_TYPE_AVAILABLE);
 
   /* Pop up the accounts dialog if we don't have any account */
@@ -540,13 +613,13 @@ chatroom_manager_ready_cb (EmpathyChatroomManager *chatroom_manager,
 }
 
 static void
-empathy_idle_set_auto_away_cb (GSettings *gsettings,
+empathy_presence_manager_set_auto_away_cb (GSettings *gsettings,
 				const gchar *key,
 				gpointer user_data)
 {
-	EmpathyIdle *idle = user_data;
+	EmpathyPresenceManager *presence_mgr = user_data;
 
-	empathy_idle_set_auto_away (idle,
+	empathy_presence_manager_set_auto_away (presence_mgr,
       g_settings_get_boolean (gsettings, key));
 }
 
@@ -554,7 +627,6 @@ static void
 empathy_app_constructed (GObject *object)
 {
   EmpathyApp *self = (EmpathyApp *) object;
-  GError *error = NULL;
   gboolean chatroom_manager_ready;
   gboolean autoaway;
 
@@ -572,16 +644,17 @@ empathy_app_constructed (GObject *object)
   notify_init (_(PACKAGE_NAME));
 
   /* Setting up Idle */
-  self->idle = empathy_idle_dup_singleton ();
+  self->presence_mgr = empathy_presence_manager_dup_singleton ();
 
   self->gsettings = g_settings_new (EMPATHY_PREFS_SCHEMA);
   autoaway = g_settings_get_boolean (self->gsettings, EMPATHY_PREFS_AUTOAWAY);
 
   g_signal_connect (self->gsettings,
       "changed::" EMPATHY_PREFS_AUTOAWAY,
-      G_CALLBACK (empathy_idle_set_auto_away_cb), self->idle);
+      G_CALLBACK (empathy_presence_manager_set_auto_away_cb),
+      self->presence_mgr);
 
-  empathy_idle_set_auto_away (self->idle, autoaway);
+  empathy_presence_manager_set_auto_away (self->presence_mgr, autoaway);
 
   /* Setting up Connectivity */
   self->connectivity = empathy_connectivity_dup_singleton ();
@@ -602,15 +675,6 @@ empathy_app_constructed (GObject *object)
 
   migrate_config_to_xdg_dir ();
 
-  /* Setting up UI */
-  self->window = empathy_main_window_dup ();
-  gtk_widget_show (self->window);
-  self->icon = empathy_status_icon_new (GTK_WINDOW (self->window),
-      self->start_hidden);
-
-  /* Chat manager */
-  self->chat_manager = empathy_chat_manager_dup_singleton ();
-
   /* Logging */
   self->log_manager = tpl_log_manager_dup_singleton ();
 
@@ -628,46 +692,21 @@ empathy_app_constructed (GObject *object)
           self->account_manager);
     }
 
-  /* Create the FT factory */
-  self->ft_factory = empathy_ft_factory_dup_singleton ();
-  g_signal_connect (self->ft_factory, "new-ft-handler",
-      G_CALLBACK (new_ft_handler_cb), NULL);
-  g_signal_connect (self->ft_factory, "new-incoming-transfer",
-      G_CALLBACK (new_incoming_transfer_cb), NULL);
-
-  if (!empathy_ft_factory_register (self->ft_factory, &error))
-    {
-      g_warning ("Failed to register FileTransfer handler: %s", error->message);
-      g_error_free (error);
-    }
-
   /* Location mananger */
 #ifdef HAVE_GEOCLUE
   self->location_manager = empathy_location_manager_dup_singleton ();
 #endif
+
+  self->activated = FALSE;
+  self->ft_factory = NULL;
+  self->window = NULL;
 }
 
 int
 main (int argc, char *argv[])
 {
   EmpathyApp *app;
-  GError *error = NULL;
-  GOptionContext *optcontext;
-  gboolean no_connect = FALSE, start_hidden = FALSE;
-  GOptionEntry options[] = {
-      { "no-connect", 'n',
-        0, G_OPTION_ARG_NONE, &no_connect,
-        N_("Don't connect on startup"),
-        NULL },
-      { "start-hidden", 'h',
-        0, G_OPTION_ARG_NONE, &start_hidden,
-        N_("Don't display the contact list or any other dialogs on startup"),
-        NULL },
-      { "version", 'v',
-        G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, show_version_cb,
-        NULL, NULL },
-      { NULL }
-  };
+  gint retval;
 
   g_thread_init (NULL);
   g_type_init ();
@@ -677,30 +716,20 @@ main (int argc, char *argv[])
 #endif
 
   empathy_init ();
-
-  optcontext = g_option_context_new (N_("- Empathy IM Client"));
-  g_option_context_add_group (optcontext, gtk_get_option_group (TRUE));
-  g_option_context_add_main_entries (optcontext, options, GETTEXT_PACKAGE);
-
-  if (!g_option_context_parse (optcontext, &argc, &argv, &error)) {
-    g_print ("%s\nRun '%s --help' to see a full list of available command line options.\n",
-        error->message, argv[0]);
-    g_warning ("Error in empathy init: %s", error->message);
-    return EXIT_FAILURE;
-  }
-
-  g_option_context_free (optcontext);
-
+  gtk_init (&argc, &argv);
   empathy_gtk_init ();
 
-  app = empathy_app_new (argc, (const gchar * const *) argv,
-      no_connect, start_hidden);
+  app = g_object_new (EMPATHY_TYPE_APP,
+      "application-id", EMPATHY_DBUS_NAME,
+      "flags", G_APPLICATION_HANDLES_COMMAND_LINE,
+      NULL);
 
-  gtk_application_run (GTK_APPLICATION (app));
+  retval = g_application_run (G_APPLICATION (app), argc, argv);
 
   notify_uninit ();
   xmlCleanupParser ();
 
   g_object_unref (app);
-  return EXIT_SUCCESS;
+
+  return retval;
 }
